package com.hz.pm.api.common.statemachine.builder.impl;

import com.hz.pm.api.common.statemachine.builder.BaseStateMachineBuilder;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderStatus;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.StateMachineContext;
import org.springframework.statemachine.StateMachineException;
import org.springframework.statemachine.StateMachinePersist;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineBuilder;
import org.springframework.statemachine.persist.DefaultStateMachinePersister;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.statemachine.support.DefaultStateMachineContext;
import org.springframework.stereotype.Component;

import java.util.EnumSet;

/**
 * <p>
 * 标段状态机
 * </p>
 *
 * @author WendyYang
 * @since 16:38 2024/2/24
 */
@Slf4j
@Component
@EnableStateMachine(name = TenderStateMachineBuilderImpl.MACHINE_ID)
@RequiredArgsConstructor
public class TenderStateMachineBuilderImpl implements BaseStateMachineBuilder<Purchase, TenderStateChangeEvent, TenderStatus> {

    public static final String MACHINE_ID = "tenderStateMachine";

    private final BeanFactory beanFactory;

    @Override
    public StateMachine<TenderStatus, TenderStateChangeEvent> build() throws StateMachineException {
        StateMachine<TenderStatus, TenderStateChangeEvent> stateMachine = build(beanFactory);
        log.info("状态机ID：" + stateMachine.getId());
        stateMachine.start();
        return stateMachine;
    }

    @Override
    public StateMachine<TenderStatus, TenderStateChangeEvent> build(BeanFactory beanFactory) throws StateMachineException {
        try {
            return buildStateMachine(beanFactory);
        } catch (Exception e) {
            throw new StateMachineException("状态机构建失败", e);
        }
    }

    private StateMachine<TenderStatus, TenderStateChangeEvent> buildStateMachine(BeanFactory factory) throws Exception {
        StateMachineBuilder.Builder<TenderStatus, TenderStateChangeEvent> builder = StateMachineBuilder.builder();
        builder.configureConfiguration()
                .withConfiguration()
                .machineId(MACHINE_ID)
                .beanFactory(factory);

        builder.configureStates()
                .withStates()
                .initial(TenderStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .states(EnumSet.allOf(TenderStatus.class));

        builder.configureTransitions()
                // 填写采购&合同信息
                .withExternal()
                .source(TenderStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .target(TenderStatus.TO_BE_SUBMIT_OPERATION_PLAN)
                .event(TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .and()
                // 填写实施计划
                .withExternal()
                .source(TenderStatus.TO_BE_SUBMIT_OPERATION_PLAN)
                .target(TenderStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO)
                .event(TenderStateChangeEvent.SUBMIT_OPERATION_PLAN)
                .and()
                // 上传初验材料
                .withExternal()
                .source(TenderStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO)
                .target(TenderStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY)
                .event(TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES)
                .and()
                // 终验申报
                .withExternal()
                .source(TenderStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY)
                .target(TenderStatus.ON_FINALLY_INSPECTED_APPLY)
                .event(TenderStateChangeEvent.SUBMIT_FINALLY_INSPECTED)
                .and()
                // 终验申报重新提交
                .withExternal()
                .source(TenderStatus.FINALLY_INSPECTED_FAILED)
                .target(TenderStatus.ON_FINALLY_INSPECTED_APPLY)
                .event(TenderStateChangeEvent.RESUBMIT_FINALLY_INSPECTED)
                .and()
                // 终验申报通过
                .withExternal()
                .source(TenderStatus.ON_FINALLY_INSPECTED_APPLY)
                .target(TenderStatus.FINALLY_INSPECTED_PASSED)
                .event(TenderStateChangeEvent.FINALLY_INSPECTED_PASSED)
                .and()
                // 终验申报拒绝
                .withExternal()
                .source(TenderStatus.ON_FINALLY_INSPECTED_APPLY)
                .target(TenderStatus.FINALLY_INSPECTED_FAILED)
                .event(TenderStateChangeEvent.FINALLY_INSPECTED_FAILED)
                .and();
        return builder.build();
    }

    @Override
    @Bean(name = "tenderStatePersister")
    public StateMachinePersister<TenderStatus, TenderStateChangeEvent, Purchase> stateMachinePersister() {
        return new DefaultStateMachinePersister<>(new StateMachinePersist<TenderStatus, TenderStateChangeEvent, Purchase>() {
            @Override
            public void write(StateMachineContext<TenderStatus, TenderStateChangeEvent> context, Purchase contextObj) {
                log.info("当前项目为：{} ==> {}", contextObj.getProjectId(), contextObj.getId());
            }

            @Override
            public StateMachineContext<TenderStatus, TenderStateChangeEvent> read(Purchase contextObj) {
                return new DefaultStateMachineContext<>(TenderStatus.getNoNull(contextObj.getStatus()),
                        null, null, null, null, MACHINE_ID);
            }
        });
    }

}
